#!/usr/bin/perl -T
#$Id: set_gas.cgi,v 1.29 2020/11/23 08:24:56 sqko Exp $
############################################################
############################################################
###                                                      ###
###   Originally designed by Soeren Koch                 ###
###                                                      ###
###   Impelemented by Soeren Koch                        ###
###                                                      ###
###   Copyright by Soeren Koch January 2011              ###
###                                                      ###
############################################################
############################################################

use strict;
use warnings;
use RFC::Main;
use RFC::Rig;
use RFC::RFCCGI qw(:cgi :html);
use RFC::Header;
use CGI qw(:standard :netscape);

my $r = param('rig');
unless ($r)
{
    print header();
    print "No rig specified!\n";
    exit;
}
my $rig = RFC::Rig->new($r);
my @d = $rig->list_devices_control('gas');
my @devs = ();
my %devlabels = ();
my $data = RFC::Cache->new();
my %errors = ();
foreach my $name (@d)
{
    my $dev = $rig->get_device('gas',$name);
    ### exclude readonly devices and non engaged gasses as well as implicitly initialised devices ###
    next if ($dev->readonly);
    next if (ref($dev) =~ /typecast/i);
    my $contr = $dev->controler;
    next if ($contr && !ref($contr));
    next if ($dev->readstring_ignore() && ref($dev) !~ /Adapter/);
    $devlabels{$name} = $dev->UIname();
    if (ref($contr))
    {
        my $m = $contr->multiplexer();
	my $g = undef;
	eval {
	    $g = $m->gas_name($data) if ($m);
	};
	if ($@)
	{
	    $errors{"Warning: could not determine gas for multiplexer ".$m->name} = $@;
	    &errorlog($@);
	}
        next if ($g && $g ne $name);
    }
    push @devs,$name;
}
### Only report multipexer errors once ###
foreach my $e (keys %errors)
{
    $rig->proglog($e);
}

if (!defined(param))
{
    &print_header("Please log in to the $name");
    print '<center>';
    RFC::RFCCGI::login();
    print '</center>';
    &print_end;
}
else
{
    #### husk at aendre accessparameter til det relevante for scriptet!! ####
    if (&login_ok())
    {
        &print_header("Set gas for rig ".$rig->name);
        if ($ENV{'HTTP_USER_AGENT'} =~ /MSIE/)
        {
	    print RFC_start_html(-title=>"Set gas for rig ".$rig->name, -style=>{src=>'/celltest/RFCcontrol.css'},-onBlur=>'initiateSelfClosing()');
        }
        else
        {
	    print RFC_start_html(-title=>"Set gas for rig ".$rig->name, -style=>{src=>'/celltest/RFCcontrol.css'},-onBlur=>'self.close()');
	}
	print '<center>';
	print '<table border=0 cellspacing=5 cellpadding=0 style="width:100%"><tr height=1>';
	print '<td width="2%"></td>';
	print '<td width="26%"></td>';
	print '<td width="35%"></td>';
	print '<td width="15%"></td>';
	print '<td width="20%"></td>';
	print '<td width="2%"></td>';
	print '</tr>';
	print start_form(-name=>'sofc');
	print '<tr><td colspan=6 class=heading> Set gas for rig ',$rig->name,'</td></tr>';
	print '<tr><td></td><th>';
	my $unit = 'L/hour';
	my $vtype = 'Flow:';
	my ($t,$r,$j,$a,$set,$ramp) = ('','','','','','');
	if (@devs > 1)
	{
	    print 'Select gas </th><th>';
	    print popup_menu(-name=>'device',-values=>['',sort @devs],-labels=>\%devlabels,-default=>'',-class=>'textfield',-force=>'1',-onchange=>'set_default()');	    
	    $j = "    var element = document.sofc.device;\n";
	    $j .= "    opener.document.gas.device.value = element[element.selectedIndex].value;\n";
	}
	else
	{
	    my $d = $rig->get_device('gas',$devs[0]);
	    $t = $d->setpoint();
	    $a = $d->read();
	    $ramp = $d->get_ramp if ($d->has_ramp);
	    if ($d->is_pressure)
	    {
		$unit = 'barA';
		$vtype = 'Pressure:';
	    }
	    $j = "    var element = document.sofc.device;\n";
	    $j .= "    opener.document.gas.device.value = document.sofc.device.value;\n";
	    print popup_menu(-name=>'device',-values=>[@devs],-labels=>\%devlabels,-default=>$devs[0],-class=>'hidden',-force=>'1');	    
	    $set = 'setTimeout("reload_val()",2000);';
	}
	print '</th><td colspan=1>';
	print '</td><td rowspan=3>';
	print button(-name=>'test',-value=>'Ok',-class=>'s3',-OnClick=>'set_relay()');
	print '</td></tr>',"\n";
	print '<tr><td></td><th id="vtype">'.$vtype.'</th><td>';
        print textfield(-name=>'flow',-value=>$t,-force=>'1',-class=>'textfield');
	print '</td><th id="unit">'.$unit.'</th></tr>',"\n";
	print '<tr><td></td><th> Actual: </th><td id="actual" style="text-align:left;">';
	print $a;
	print '</td><th id="unit2">'.$unit.'</th></tr>',"\n";
	if (@devs == 1 && $rig->get_device('gas',$devs[0])->has_ramp)
	{
	    print '<tr><td></td><th>Ramprate:</th><td>';
	    print textfield(-name=>'ramprate',-value=>$ramp,-force=>'1',-class=>'textfield');
	    print '</td><th>'.$unit.'/hour</th></tr>',"\n";	    
	}
	else
	{
	    print '<tr><td></td><th id="ramp_text"></th><td>';
	    print textfield(-id=>'ramprate',-name=>'ramprate',-value=>'',-force=>'1',-class=>'hidden');
	    print '</td><th id="unit3"></th></tr>',"\n";
	}
	print end_form;
	print <<END;

<script language=javascript>

function set_default()
{
    var element = document.sofc.device;
    var id = element.selectedIndex;
    var name = element[id].value;
    document.sofc.flow.disabled = true;
    document.sofc.ramprate.disabled = true;
    run_ajax('rig_ajax_run.cgi','&action=run_device_function&type=gas&dname='+name+'&function=setpoint',set_default_ajax);
    get_unit();
}

function get_unit()
{
    var element = document.sofc.device;
    var id = element.selectedIndex;
    var name = element[id].value;
    document.sofc.flow.disabled = true;
    document.sofc.ramprate.disabled = true;
    run_ajax('rig_ajax_run.cgi','&action=run_device_function&type=gas&dname='+name+'&function=is_pressure',set_default_unit);
}


function set_ramprate(res)
{
    var arr = res.split("\\n");
    if (arr[0] == "OK")
    {
	var has_ramp = false;
	if (arr[1] == 1)
	{
	    var has_ramp = true;
	}
	var ramp = document.getElementById("ramprate");
	var ramp_text = document.getElementById("ramp_text");
	var unit = document.getElementById("unit").innerHTML;
	var unit3 = document.getElementById("unit3");
	if (has_ramp)
	{
	    unit3.innerHTML = unit+"/hour";
	    ramp_text.innerHTML = "Ramprate:";
	    ramp.value = arr[2];
	    ramp.className = "textfield";
	}
	else
	{
	    unit3.innerHTML = "";
	    ramp_text.innerHTML = "";
	    ramp.className = "hidden";
	}
    }
    else
    {
	alert(res);
    }
    document.sofc.flow.disabled = false;
    document.sofc.ramprate.disabled = false;
}

function set_default_unit(res)
{
    var arr = res.split("\\n");
    if (arr[0] == "OK")
    {
	var p = arr[1];
	var vtype = document.getElementById("vtype");
	var unit = document.getElementById("unit");
	var unit2 = document.getElementById("unit2");
	if (p == 1)
	{
	    vtype.innerHTML = "Pressure:";
	    unit.innerHTML = "barA";
	    unit2.innerHTML = "barA";
	}
	else
	{
	    vtype.innerHTML = "Flow:";
	    unit.innerHTML = "L/hour";
	    unit2.innerHTML = "L/hour";
	}
    }
    else
    {
	alert(res);
    }
    var element = document.sofc.device;
    var id = element.selectedIndex;
    var name = element[id].value;
    run_ajax('rig_ajax_run.cgi','&action=get_ramprate&type=gas&dname='+name,set_ramprate);
}


function set_default_ajax(res)
{
    var arr = res.split("\\n");
    if (arr[0] == "OK")
    {
	document.sofc.flow.value = arr[1];	
	document.sofc.flow.disabled = false;
	document.sofc.ramprate.disabled = false;
	/* Get curent flow value*/
	var element = document.sofc.device;
	var id = element.selectedIndex;
	var name = element[id].value;
	run_ajax('rig_ajax_run.cgi','&action=run_device_function&type=gas&dname='+name+'&function=read',set_default_value_ajax);
    }
    else
    {
	alert(res);
    }
}

function set_default_value_ajax(res)
{
    var arr = res.split("\\n");
    if (arr[0] == "OK")
    {
	var value = document.getElementById("actual");
	value.innerHTML = arr[1];
	setTimeout("reload_val()",2000);
    }
    else
    {
	alert(res);
    }
}


function reload_val()
{
    var element = document.sofc.device;
    var id = element.selectedIndex;
    var name = element[id].value;
    run_ajax('rig_ajax_run.cgi','&action=run_device_function&type=gas&dname='+name+'&function=read',set_default_value_ajax);
}



function set_relay()
{
$j    
    opener.document.gas.flow.value = document.sofc.flow.value;
    opener.document.gas.ramp.value = document.sofc.ramprate.value;
    opener.set_gas_go();
    self.close();
}

$set

</script>

END

        print end_html;
    }
    else
    {
        &not_auth;
    }
}

